//*********************************************************************************/
#ifndef _OSA
#define _OSA

#define __OSA__

#define __OSAVERSION__		90115

#include "OSAcfg.h"



/************************************************************************/
/* COMPILER CONSTANT                                                    */
/************************************************************************/

#if   defined(__PICC18__)

    #define __OSAPICC18__
    #define __OSA18__

#elif defined(__18CXX)

    #define __OSAMCC18__
    #define __OSA18__

#elif defined(__C30__)

    #define __OSAMCC30__
    #define __OSA30__

#elif defined(__PCH__)


    #define __OSACCS18__
    #define __OSA18__

#elif defined(_PIC12)
    
    #define __OSAPICC12__
    #define __OSA12__

#else

    #define __OSAPICC16__
    #define __OSAPICC__
    #define __OSA16__

#endif


/************************************************************************/
/* OSA CONFIGURATION                                                    */
/************************************************************************/
#ifdef OS_ENABLE_ALL
    // undef all config constants
    #undef OS_ENABLE_TIMER
    #undef OS_ENABLE_CSEM
    #undef OS_ENABLE_QMSG
    #undef OS_ENABLE_QSMSG
    #undef OS_ENABLE_DTIMERS
    #undef OS_ENABLE_CRITICAL_SECTION

    // undef all old style config constants
    #undef OS_TIMER_ENABLE
    #undef OS_CSEM_ENABLE
    #undef OS_QMSG_ENABLE
    #undef OS_QSMSG_ENABLE
    #undef OS_DTIMERS_ENABLE
    

    // define all config constants
    #define OS_ENABLE_CSEM
    #define OS_ENABLE_QMSG
    #define OS_ENABLE_QSMSG
    #define OS_ENABLE_DTIMERS
    #define OS_ENABLE_TTIMERS
    #define OS_ENABLE_CRITICAL_SECTION

#endif



//------------------------------------------------------------------------------
// Redefining old style constants

#ifdef OS_TIMER_ENABLE
#define OS_ENABLE_TIMER                 // Define when using timers in tasks (OS_Delay and wait events with timeout)
#endif

#ifdef OS_CSEM_ENABLE
#define OS_ENABLE_CSEM                  // Define when using counting semaphores 
#endif

#ifdef OS_QMSG_ENABLE
#define OS_ENABLE_QMSG                  // Define when using queue of pointer to messages
#endif

#ifdef OS_QSMSG_ENABLE
#define OS_ENABLE_QSMSG                 // Define when using queue of simple messages
#endif

#ifdef OS_DTIMERS_ENABLE
#define OS_ENABLE_DTIMERS
#endif

#ifdef OS_DISABLE_TASK_TIMERS   
    #define OS_DISABLE_TTIMERS
#endif 




//------------------------------------------------------------------------------
// Set constants for interrupt usege
#ifdef  OS_ENABLE_INT_SERVICE       // Redefining old style constant
#define OS_ENABLE_INT_ALL
#endif

#ifdef OS_ENABLE_INT_ALL
    #define OS_ENABLE_INT_CSEM
    #define OS_ENABLE_INT_MSG
    #define OS_ENABLE_INT_SMSG
    #define OS_ENABLE_INT_QMSG
#endif


//------------------------------------------------------------------------------
// Set default system constants (if they are not set in OSAcfg.h)

#if !defined(OS_STIMERS)
#define OS_STIMERS  0
#endif

#ifndef OS_TIMERS8
    #define OS_TIMERS8 0
#endif

#ifndef OS_TIMERS16
    #define OS_TIMERS16 0
#endif

#ifndef OS_TIMERS24
    #define OS_TIMERS24 0
#endif

#ifndef OS_TIMERS32
    #define OS_TIMERS32 0
#endif

#define OS_TIMERS           (OS_TIMERS8 + OS_TIMERS16 + OS_TIMERS24 + OS_TIMERS32)

#if OS_STIMERS > 0
    #define OS_ENABLE_STIMERS
#endif



#if defined(OS_ENABLE_TIMER)
#define OS_ENABLE_TTIMERS
#endif


#ifdef OS_ENABLE_PTIMERS        // Redefine old name (ptimer) to new (dtimer)
    #define OS_ENABLE_DTIMERS
#endif

#ifdef OS_ENABLE_DTIMERS        // Enable system timer if dynamic timers used
    #define OS_ENABLE_OS_TIMER
#endif

#ifdef OS_ENABLE_TTIMERS        // Enable system timer if task timers used
    #define OS_ENABLE_OS_TIMER
#endif

#ifdef OS_ENABLE_STIMERS        // Enable system timer if static timers used
    #define OS_ENABLE_OS_TIMER
#endif

#if OS_TIMERS > 0
    #define OS_ENABLE_OS_TIMER
#endif


//------------------------------------------------------------------------------









//------------------------------------------------------------------------------


#ifdef __OSACCS18__
    #define __osa_uint8     unsigned int8
    #define __osa_uint16    unsigned int16
    #define __osa_uint32    unsigned int32
#else
    typedef unsigned char    __osa_uint8;
    typedef unsigned int     __osa_uint16;
    typedef unsigned long    __osa_uint32;
#endif

// Define integer type according to size of one RAM-word
#ifdef __OSA30__
    #define _OST_INT_TYPE   __osa_uint16
    #define _OST_INT_MASK   15
    #define _OST_INT_SHIFT  4
    #define _OST_INT_SIZE   16
#else
    #define _OST_INT_TYPE    __osa_uint8
    #define _OST_INT_MASK    7
    #define _OST_INT_SHIFT   3
    #define _OST_INT_SIZE    8
#endif







/****************************************************************************************************
 *
 * DEFINE BANKS TO ALLOCATE DATA
 *
 ****************************************************************************************************/

#if defined(OS_BANK) || defined(OS_TASKS_BANK)  || defined(OS_TIMERS_BANK)  ||  defined(OS_TIMERS8_BANK)  ||  defined(OS_TIMERS16_BANK)  || defined(OS_TIMERS24_BANK)  || defined(OS_TIMERS32_BANK)  || defined(OS_TIMEOUTS_BANK)  || defined(OS_BSEM_BANK)
#error "Constants OS_xxx_BANK are not allowed now (use OS_BANK_xxx instead, see manual (3.4))"
#endif




#ifndef OS_BANK_OS
#define OS_BANK_OS      0
#endif

#if defined(OS_BANK_TASK) && !defined(OS_BANK_TASKS)
#define OS_BANK_TASKS   OS_BANK_TASK
#endif


#ifndef OS_BANK_TASKS
#define OS_BANK_TASKS       OS_BANK_OS       // Bank for task descriptors
#endif

#ifndef OS_BANK_TIMERS
#define OS_BANK_TIMERS      OS_BANK_OS
#endif

#ifndef OS_BANK_TIMERS8 
#define OS_BANK_TIMERS8     OS_BANK_TIMERS       // Bank for old type static timers8
#endif

#ifndef OS_BANK_TIMERS16
#define OS_BANK_TIMERS16    OS_BANK_TIMERS       // Bank for old type static timers16
#endif

#ifndef OS_BANK_TIMERS24
#define OS_BANK_TIMERS24    OS_BANK_TIMERS       // Bank for old type static timers24
#endif

#ifndef OS_BANK_TIMERS32
#define OS_BANK_TIMERS32    OS_BANK_TIMERS       // Bank for old type static timers32
#endif

#ifndef OS_BANK_TIMEOUTS
#define OS_BANK_TIMEOUTS    OS_BANK_TIMERS
#endif

#ifndef OS_BANK_STIMERS
#define OS_BANK_STIMERS     OS_BANK_TIMERS      // Bank for static timers
#endif

#if defined(OS_BANK_BSEMS) && !defined(OS_BANK_BSEM)
#define OS_BANK_BSEM   OS_BANK_BSEMS
#endif

#ifndef OS_BANK_BSEM
#define OS_BANK_BSEM      OS_BANK_OS       // Bank for binary semaphores
#endif




    
    
/************************************************************************/
/* DEFINE 'BANK' KEYWORDS FOR DIFFERENT COMPILERS                       */
/************************************************************************/
    
//------------------------------------------------------------------------------
#if defined(__OSAPICC18__)
    #define NEAR    near
    #define bank0   near
    #define bank1   bdata
    #define bank2   bdata
    #define bank3   bdata
    #define __osa30_near
#endif
//------------------------------------------------------------------------------
#if defined (__OSAMCC18__)
    #define NEAR    near
    #define bank0   near
    #define bank1
    #define bank2
    #define bank3
    #define bit     _OST_INT_TYPE
    #define __osa30_near
#endif
//------------------------------------------------------------------------------
#if defined (__OSAMCC30__)
    #define NEAR
    #define bank0
    #define bank1
    #define bank2
    #define bank3
    #define bit             _OST_INT_TYPE
    #define __osa30_near    __attribute__((near))
#endif
//------------------------------------------------------------------------------
#if defined (__OSACCS18__)
    #define NEAR
    #define bank0
    #define bank1
    #define bank2
    #define bank3
    #define bit     _OST_INT_TYPE
    #define __osa30_near
#endif
//------------------------------------------------------------------------------
#if defined(__OSAPICC__)
    #define NEAR    near
    #define bank0   near
    #define __osa30_near
#endif
//------------------------------------------------------------------------------
#if defined(__OSAPICC12__)
    #define NEAR    near
    #define bank0   near
    #define __osa30_near
#endif


//------------------------------------------------------------------------------
#if   OS_BANK_OS == 0
#define OS_BANK bank0
#elif OS_BANK_OS == 1
#define OS_BANK bank1
#elif OS_BANK_OS == 2
#define OS_BANK bank2
#elif OS_BANK_OS == 3
#define OS_BANK bank3
#else
    #error  "Incorrect OS_BANK_OS value (should be 0, 1, 2 or 3)"
#endif

//------------------------------------------------------------------------------
#if   OS_BANK_TASKS == 0
#define OS_TASKS_BANK bank0
#elif OS_BANK_TASKS == 1
#define OS_TASKS_BANK bank1
#elif OS_BANK_TASKS == 2
#define OS_TASKS_BANK bank2
#elif OS_BANK_TASKS == 3
#define OS_TASKS_BANK bank3
#else
    #error  "Incorrect OS_BANK_TASKS value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_TIMERS == 0
#define OS_TIMERS_BANK bank0
#elif OS_BANK_TIMERS == 1
#define OS_TIMERS_BANK bank1
#elif OS_BANK_TIMERS == 2
#define OS_TIMERS_BANK bank2
#elif OS_BANK_TIMERS == 3
#define OS_TIMERS_BANK bank3
#else
    #error  "Incorrect OS_BANK_TIMERS value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_STIMERS == 0
#define OS_STIMERS_BANK bank0
#elif OS_BANK_STIMERS == 1
#define OS_STIMERS_BANK bank1
#elif OS_BANK_STIMERS == 2
#define OS_STIMERS_BANK bank2
#elif OS_BANK_STIMERS == 3
#define OS_STIMERS_BANK bank3
#else
    #error  "Incorrect OS_BANK_STIMERS value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_TIMERS8 == 0
#define OS_TIMERS8_BANK bank0
#elif OS_BANK_TIMERS8 == 1
#define OS_TIMERS8_BANK bank1
#elif OS_BANK_TIMERS8 == 2
#define OS_TIMERS8_BANK bank2
#elif OS_BANK_TIMERS8 == 3
#define OS_TIMERS8_BANK bank3
#else
    #error  "Incorrect OS_BANK_TIMERS8 value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_TIMERS16 == 0
#define OS_TIMERS16_BANK bank0
#elif OS_BANK_TIMERS16 == 1
#define OS_TIMERS16_BANK bank1
#elif OS_BANK_TIMERS16 == 2
#define OS_TIMERS16_BANK bank2
#elif OS_BANK_TIMERS16 == 3
#define OS_TIMERS16_BANK bank3
#else
    #error  "Incorrect OS_BANK_TIMERS16 value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_TIMERS24 == 0
#define OS_TIMERS24_BANK bank0
#elif OS_BANK_TIMERS24 == 1
#define OS_TIMERS24_BANK bank1
#elif OS_BANK_TIMERS24 == 2
#define OS_TIMERS24_BANK bank2
#elif OS_BANK_TIMERS24 == 3
#define OS_TIMERS24_BANK bank3
#else
    #error  "Incorrect OS_BANK_TIMERS24 value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_TIMERS32 == 0
#define OS_TIMERS32_BANK bank0
#elif OS_BANK_TIMERS32 == 1
#define OS_TIMERS32_BANK bank1
#elif OS_BANK_TIMERS32 == 2
#define OS_TIMERS32_BANK bank2
#elif OS_BANK_TIMERS32 == 3
#define OS_TIMERS32_BANK bank3
#else
    #error  "Incorrect OS_BANK_TIMERS32 value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_TIMEOUTS == 0
#define OS_TIMEOUTS_BANK bank0
#elif OS_BANK_TIMEOUTS == 1
#define OS_TIMEOUTS_BANK bank1
#elif OS_BANK_TIMEOUTS == 2
#define OS_TIMEOUTS_BANK bank2
#elif OS_BANK_TIMEOUTS == 3
#define OS_TIMEOUTS_BANK bank3
#else
    #error  "Incorrect OS_BANK_TIMEOUTS value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
#if   OS_BANK_BSEM == 0
#define OS_BSEM_BANK bank0
#elif OS_BANK_BSEM == 1
#define OS_BSEM_BANK bank1
#elif OS_BANK_BSEM == 2
#define OS_BSEM_BANK bank2
#elif OS_BANK_BSEM == 3
#define OS_BSEM_BANK bank3
#else
    #error  "Incorrect OS_BANK_BSEM value (should be 0, 1, 2 or 3)"
#endif
//------------------------------------------------------------------------------
















//______________________________________________________________________________
//******************************************************************************
// Number of task descriptors (maximum active tasks at one time)
#if OS_TASKS == 0
    #undef OS_TASKS
#endif

#if !defined(OS_TASKS)
    #define OS_TASKS    1       // We need at least 1 task
#endif






/************************************************************************/
/* DEFINE TYPE OF TIMERS                                                */
/************************************************************************/

#if defined(OST_TIMER)
#error "OST_TlMER is not supported! Use OS_TIMER_SIZE instead (see manual 3.3)"
#endif

#if !defined(OS_TIMER_SIZE)
    #define OS_TIMER_SIZE   2
#endif

#if   OS_TIMER_SIZE == 1
    #define OST_TIMER __osa_uint8
#elif OS_TIMER_SIZE == 2
    #define OST_TIMER __osa_uint16
#elif OS_TIMER_SIZE == 4
    #define OST_TIMER __osa_uint32
#endif


//------------------------------------------------------------------------------
// Define type of static timer's counter 
#if !defined(OS_STIMER_SIZE)
#define OS_STIMER_SIZE  OS_TIMER_SIZE
#endif

#if   OS_STIMER_SIZE == 1
    #define _OST_STIMER __osa_uint8
#elif OS_STIMER_SIZE == 2
    #define _OST_STIMER __osa_uint16
#elif OS_STIMER_SIZE == 4
    #define _OST_STIMER __osa_uint32
#endif

//------------------------------------------------------------------------------
// Define type of dynamic timer's counter

#ifdef OS_PTIMER_SIZE        // Redefine old name (ptimer) to new (dtimer)
    #define OS_DTIMER_SIZE OS_PTIMER_SIZE
#endif

#if !defined(OS_DTIMER_SIZE)
#define OS_DTIMER_SIZE  OS_TIMER_SIZE
#endif

#if   OS_DTIMER_SIZE == 1
    #define _OST_DTIMER __osa_uint8
#elif OS_DTIMER_SIZE == 2
    #define _OST_DTIMER __osa_uint16
#elif OS_DTIMER_SIZE == 4
    #define _OST_DTIMER __osa_uint32
#endif

//------------------------------------------------------------------------------
// Define type of task's timers
#if !defined(OS_TTIMER_SIZE)
#define OS_TTIMER_SIZE  OS_TIMER_SIZE
#endif

#if   OS_TTIMER_SIZE == 1
    #define _OST_TTIMER __osa_uint8
#elif OS_TTIMER_SIZE == 2
    #define _OST_TTIMER __osa_uint16
#elif OS_TTIMER_SIZE == 4
    #define _OST_TTIMER __osa_uint32
#endif





























//______________________________________________________________________________
//******************************************************************************
// S Y S T E M   D A T A   T Y P E S


//------------------------------------------------------------------------------
// Pointer to message

#ifndef OST_MSG
    #define OST_MSG   void*
#endif

//------------------------------------------------------------------------------
// Simple byte message

#ifndef OST_SMSG
    #define OST_SMSG    unsigned char
#endif


//------------------------------------------------------------------------------
/************************************************************************/
/* INCLUDE PROCESSOR SPECIFIC DEFINITIONS                               */
/************************************************************************/
#if defined(__OSAPICC18__)
    #include "port\osa_picc18.h"         // HT-PICC18
#elif defined(__OSAMCC18__)
    #include "port\osa_mcc18.h"          // MPLAB C18
#elif defined(__OSAMCC30__)
    #include "port\osa_mcc30.h"          // MPLAB C30
#elif defined(__OSACCS18__)
    #include "port\osa_ccs18.h"          // CCS-PICC (for PIC18 only)
#elif defined(__OSAPICC12__)
    #include "port\osa_picc12.h"         // HT_PICC for 12-bit PICs
#elif defined(__OSAPICC__)
    #include "port\osa_picc16.h"         // HT_PICC
#else
    #error "Unknown compiler!"
#endif

//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// Pointer to message descriptor
typedef struct {
    _OST_INT_TYPE   status;     // State (=1 - busy / =0 - free)
    OST_MSG         msg;        // Pointer to message body
} OST_PMSG;





//------------------------------------------------------------------------------
// Message's queues

// Queue's control block
typedef struct {                 
    _OST_INT_TYPE   cSize;      // Queue size
    _OST_INT_TYPE   cFilled;    // Number of filled elements
    _OST_INT_TYPE   cBegin;     // First element in buffer
} OST_QUEUE_CONTROL;



// Descriptor of queue of pointers to messages
typedef struct {
    OST_QUEUE_CONTROL Q;
    OST_MSG *pMsg;              // Pointer to queue buffer
} OST_QMSG;


// Descriptor of queue of simple messages
typedef struct {
    OST_QUEUE_CONTROL Q;
    OST_SMSG *pSMsg;            // Pointer to queue buffer
} OST_QSMSG;


//------------------------------------------------------------------------------
// Flags
#define OST_FLAG    __osa_uint8
#define OST_FLAG8   __osa_uint8
#define OST_FLAG16  __osa_uint16
#define OST_FLAG32  __osa_uint32



/*************************************************************************************************************************/
//------------------------------------------------------------------------------





//______________________________________________________________________________

//------------------------------------------------------------------------------
// System state flags
// !Do not change positions of theese bits.
// For MCC30 bits cIPL_Temp must occupy 5, 6 and 7 positions
typedef struct {

    /*0*/_OST_INT_TYPE bEventError:1;       // Event error
    /*1*/_OST_INT_TYPE bError:1;            // Task create error
    /*2*/_OST_INT_TYPE bInCriticalSection:1;// One of task is in critical section
    /*3*/_OST_INT_TYPE bCheckingTasks:1;    // For OS_Sched: ==1 - best task searching
                                            //               ==0 - best task found
    /*4*/_OST_INT_TYPE bBestTaskFound:1;    //     


    #if defined(__OSA30__)
    /*5..7*/    _OST_INT_TYPE cIPL_Temp:3;  // Saved InterruptPriotityLevel
                                            // when entering critical section
    #endif
                                            // 
    /* MCC - 8*/_OST_INT_TYPE bTimeout:1;// Event waiting timeout 
    /*- 5*/
    //..........................................................................................
    #if defined(__OSAMCC30__)
        /*9*/ _OST_INT_TYPE bTaskReplaced:1;    // Tells that task was replaced (with OS_ReplaceTask)
                                                // so we don't need to save stack pointer for MCC30
    
        /*10*/_OST_INT_TYPE bOSRetPoint_LNK:1;      // Theese 5 bit variables contains information
        /*11*/_OST_INT_TYPE bOSRetPointNoSave_LNK:1;// about enablity "lnk" instruction in the
        /*12*/_OST_INT_TYPE bOSClearReady_LNK:1;    // begin of 5 switching context routines.
        /*13*/_OST_INT_TYPE bOSClearReadySetCanContinue_LNK:1;
        /*14*/_OST_INT_TYPE bOSCheckEvent_LNK:1;
    #endif
    
    //..........................................................................................
    #if defined(__OSA12__)                  // For PIC12: Two user purpose bits
        /*6*/_OST_INT_TYPE bUser0:1;
        /*7*/_OST_INT_TYPE bUser1:1;
    #endif
    //..........................................................................................
    #if defined(__OSA16__)
        /*6*/_OST_INT_TYPE bGIE_CTemp:1;    // Saved GIE while in critical section
    #endif

    #if defined(__OSA18__)
        /*6*/_OST_INT_TYPE bGIE_CTemp:1;    // Saved GIEH while in critical section
        /*7*/_OST_INT_TYPE bGIEL_CTemp:1;   // Saved GIEL while in critical section
    #endif

} OST_SYSTEM_FLAGS;



//------------------------------------------------------------------------------
// Task state flags
// !Do not change placement of theese bits
typedef struct {
    /*0..2*/_OST_INT_TYPE cPriority:3;      // Task priority (0..7)
    /*3*/   _OST_INT_TYPE bReady:1;         // Ready to execute
    /*4*/   _OST_INT_TYPE bDelay:1;         // Timer active: OS_Delay (bCanContinue=0) 
                                            // or event waiting with timeout (bCanContinue=1)
    /*5*/   _OST_INT_TYPE bCanContinue:1;
    /*6*/   _OST_INT_TYPE bEnable:1;        // Descriptor is busy by task
    /*7*/   _OST_INT_TYPE bPaused:1;        // Task paused (it is still active, but can 
                                            // became ready only from extenal task)
} OST_TASK_STATE;





//------------------------------------------------------------------------------
// Task control block

typedef struct {
    OST_TASK_STATE State;
    OST_CODE_POINTER pTaskPointer;      // ROM pointer 
    #ifdef __OSA18__
        unsigned char  cTempFSR1;       // Saved FSR1 (for MCC18)
    #endif
    #ifdef __OSA30__
        __osa_uint16    W8_Temp;
        __osa_uint16    W9_Temp;
        __osa_uint16    W10_Temp;
        __osa_uint16    W11_Temp;
        __osa_uint16    W12_Temp;
        __osa_uint16    W13_Temp;
        __osa_uint16    W14_Temp;
        __osa_uint16    W15_Temp;
    #endif
    #ifdef OS_ENABLE_TTIMERS
        _OST_TTIMER Timer;              // Timer counter for delays
    #endif
} OST_TCB;   // Task Control Block


#define OS_WORST_PRIORITY       7       // Lowest priority
#define OS_BEST_PRIORITY        0       // Highest priority



//------------------------------------------------------------------------------
#ifdef OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------
// Dynamic timer's state flags
// !Do not change placement of variables
typedef struct {
    _OST_INT_TYPE   bTimeout:1;         // Overflow occured
    _OST_INT_TYPE   bActive:1;          // Timer is active (was created by OS_CreateDTimer)
                                        // (For system variable _OS_DTimers.Flags allways = 1)
    _OST_INT_TYPE   bRun:1;             // = 1 - Timer is counnting
                                        // = 0 - Timer is stopped
    _OST_INT_TYPE   bNextEnable:1;      // = 0 - last timer in list, =1 - there is next timer
} OST_DTIMER_FLAGS;

//------------------------------------------------------------------------------
// Header of list of dynamic timers
typedef struct
{
    OST_DTIMER_FLAGS Flags;
    void *           Next;
} OST_DTIMER_CB;

//------------------------------------------------------------------------------
// Dynamic timer's control block
typedef struct
{
    OST_DTIMER_FLAGS Flags;     // Timer's state flags
    void *           Next;      // Pointer to next timer in list
    _OST_DTIMER      Timer;     // Counter
} OST_DTIMER;


//------------------------------------------------------------------------------
#endif  // OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------























//______________________________________________________________________________
//******************************************************************************
// SYSTEM VARIABLES
#define OST_TCB_POINTER     OST_TASK_POINTER


extern NEAR OST_SYSTEM_FLAGS    OS_Flags    __osa30_near;
extern NEAR _OST_INT_TYPE       _OS_temp    __osa30_near;   // Addition temp variable
                                                            // In some system services used for saving GIE
#ifndef __OSA12__
extern NEAR _OST_INT_TYPE       _OS_tempH   __osa30_near;   // Addition temp variable
#endif


#if defined(__OSAMCC30__) || defined(__OSAPICC12__) || defined(__OSAPICC18__)
extern NEAR OST_TASK_STATE      OS_state    __osa30_near;    // Work with task state making through this variable
#endif                                                       // 

extern OST_TASK_POINTER NEAR    _OS_CurTask __osa30_near;    // Pointer to current task


#if !defined(OS_DISABLE_PRIORITY)
    extern OS_BANK _OST_INT_TYPE       _OS_best_priority   __osa30_near;  // best priority while searching best task
    extern OST_TASK_POINTER OS_BANK    _OS_BestTask        __osa30_near;  // Pointer to best task
    extern OST_TASK_POINTER OS_BANK    _OS_LastTask        __osa30_near;  // Pointer to last task
#endif

extern OS_TASKS_BANK    OST_TCB             OS_TaskVars[OS_TASKS]   __osa30_near;


#if OS_TIMERS24 > 0
    extern OS_BANK _OST_INT_TYPE OS_Ticks    __osa30_near;
#endif

#ifdef OS_ENABLE_DTIMERS
    extern OS_BANK OST_DTIMER_CB _OS_DTimers    __osa30_near; // Pointer to list of dynamic timers
    #if defined(__OSAMCC30__)
        extern OS_BANK OST_DTIMER *  _OS_DTimerWork __osa30_near; // For work with list in interrupt
    #endif
#endif



//______________________________________________________________________________
//******************************************************************************
// Old style static timers

#ifndef OS_TIMERS8
    #define OS_TIMERS8 0
#endif

#ifndef OS_TIMERS16
    #define OS_TIMERS16 0
#endif

#ifndef OS_TIMERS24
    #define OS_TIMERS24 0
#endif

#ifndef OS_TIMERS32
    #define OS_TIMERS32 0
#endif




#define _OS_TIMER8_POS      (0)
#define _OS_TIMER16_POS     (_OS_TIMER8_POS  + OS_TIMERS8)
#define _OS_TIMER24_POS     (_OS_TIMER16_POS + OS_TIMERS16)
#define _OS_TIMER32_POS     (_OS_TIMER24_POS + OS_TIMERS24)

#if OS_TIMERS > 0
    extern OS_TIMEOUTS_BANK _OST_INT_TYPE OS_Timeouts[] __osa30_near;
#endif

#if OS_TIMERS32 > 0
    extern OS_TIMERS32_BANK __osa_uint32 OS_Timers32[] __osa30_near;   // 32-bit timers
#endif

#if OS_TIMERS24 > 0
    extern OS_TIMERS24_BANK __osa_uint16 OS_Timers24[] __osa30_near;   // 24-bit timers
#endif

#if OS_TIMERS16 > 0
    extern OS_TIMERS16_BANK __osa_uint16 OS_Timers16[] __osa30_near;   // 16-bit timers
#endif

#if OS_TIMERS8  > 0
    extern OS_TIMERS8_BANK __osa_uint8 OS_Timers8[] __osa30_near;      // 8-bit timers
#endif



//------------------------------------------------------------------------------
// Static timers
#if OS_STIMERS > 0
    extern OS_STIMERS_BANK _OST_STIMER  OS_STimers[] __osa30_near;
#endif


//______________________________________________________________________________
//******************************************************************************
// Binary semaphores

#ifndef OS_BSEMS
#define OS_BSEMS    0
#endif

#if OS_BSEMS > 0 
    extern OS_BSEM_BANK _OST_INT_TYPE OS_BSems[] __osa30_near;
#endif






//______________________________________________________________________________
//******************************************************************************
// Counting semaphores
#ifdef OS_ENABLE_CSEM

    #if !defined(OS_CSEM_SIZE)
        #define OS_CSEM_SIZE    1
    #endif

    #if     OS_CSEM_SIZE == 1
        typedef __osa_uint8 OST_CSEM;
    #elif   OS_CSEM_SIZE == 2
        typedef __osa_uint16 OST_CSEM;
    #elif   OS_CSEM_SIZE == 4
        typedef __osa_uint32 OST_CSEM;
    #else
        #error "Incorrect OS_CSEM_SIZE value (m.b. only 1, 2 or 4)! (see OSAcfg.h)"
    #endif

    extern void _OS_SignalCSem(OST_CSEM *pCSem);

#endif

#if !defined(OS_CSEM_SIZE)
    #define OS_CSEM_SIZE    0
#endif



















/*************************************************************************************************************************/
//------------------------------------------------------------------------------
// Internal system functions prototypes

#ifdef OS_ENABLE_TTIMERS
    extern void _OS_InitDelay(_OST_TTIMER Delay);

#endif


#ifdef OS_ENABLE_OS_TIMER

    #ifndef OS_USE_INLINE_TIMER
        extern void OS_Timer (void);
    #else
        #define OS_Timer()  __OS_TimerInline(0)
    #endif

#endif

#ifdef OS_ENABLE_DTIMERS
    void _OS_CreateDTimer (OST_DTIMER *dtimer);
    void _OS_DeleteDTimer (OST_DTIMER *dtimer);
    #define OS_CreateDTimer(dtimer)     _OS_CreateDTimer(&dtimer)
    #define OS_DeleteDTimer(dtimer)     _OS_DeleteDTimer(&dtimer)
#endif


extern void _OS_StopTask (void);                // Stop current task

#if defined(__OSAMCC30__)
    extern void _OS_CreateTask(_OST_INT_TYPE priority, OST_CODE_POINTER TaskAddr);
#else
    extern bit  _OS_CreateTask(_OST_INT_TYPE priority, OST_CODE_POINTER TaskAddr);
#endif

//------------------------------------------------------------------------------
// System variables initialization
extern void  OS_Init (void);


// Critical sections
extern void OS_EnterCriticalSection (void);
extern void OS_LeaveCriticalSection (void);



//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#define OS_WAIT_NO_TIMEOUT          0
#define OS_TASK_ENABLED             1
#define OS_TASK_DISABLED            0








//______________________________________________________________________________
////////////////////////////////////////////////////////////////////////////////













/*************************************************************************************************************************/
















/*************************************************************************************************************************/
/*************************************************************************************************************************/

/************************************************************************/
/*                                                                      */
/* Internal macro for work with state flags                             */
/*                                                                      */
/************************************************************************/

#if defined(__OSAMCC30__) || defined(__OSAPICC12__) || defined(__OSAPICC18__)

    // For PICC12, PICC18 and MCC30
    #define _OS_bTaskReady              OS_state.bReady
    #define _OS_bTaskCanContinue        OS_state.bCanContinue
    #define _OS_bTaskDelay              OS_state.bDelay
    #define _OS_bTaskTimeout            (!OS_state.bDelay)
    #define _OS_bTaskEnable             OS_state.bEnable
    #define _OS_cTaskPriority           OS_state.cPriority
    #define _OS_bTaskPaused             OS_state.bPaused
    
#elif defined(__OSACCS18__)
    // For CCS 

    #define _OS_bTaskReady              ((OST_TASK_STATE)_indf).bReady
    #define _OS_bTaskCanContinue        ((OST_TASK_STATE)_indf).bCanContinue
    #define _OS_bTaskDelay              ((OST_TASK_STATE)_indf).bDelay
    #define _OS_bTaskTimeout            (!(((OST_TASK_STATE)_indf).bDelay)) 
    #define _OS_bTaskEnable             ((OST_TASK_STATE)_indf).bEnable
    #define _OS_cTaskPriority           ((OST_TASK_STATE)_indf).cPriority
    #define _OS_bTaskPaused             ((OST_TASK_STATE)_indf).bPaused

#else

    // For MCC18 and PICC16

    #define _OS_bTaskReady              ((near OST_TASK_STATE*)(&_indf))->bReady
    #define _OS_bTaskCanContinue        ((near OST_TASK_STATE*)(&_indf))->bCanContinue
    #define _OS_bTaskDelay              ((near OST_TASK_STATE*)(&_indf))->bDelay
    #define _OS_bTaskTimeout            (!((near OST_TASK_STATE*)(&_indf))->bDelay)
    #define _OS_bTaskEnable             ((near OST_TASK_STATE*)(&_indf))->bEnable
    #define _OS_cTaskPriority           ((near OST_TASK_STATE*)(&_indf))->cPriority
    #define _OS_bTaskPaused             ((near OST_TASK_STATE*)(&_indf))->bPaused

#endif




//------------------------------------------------------------------------------
// Include services:

#include "service\osa_system.h"     // System services
#include "service\osa_bsem.h"       // Binary semaphores
#include "service\osa_csem.h"       // Counting semaphores
#include "service\osa_flag.h"       // Flags
#include "service\osa_stimer.h"     // Static timers
#include "service\osa_stimer_old.h" // Old style static timers
#include "service\osa_dtimer.h"     // Dynamic timers
#include "service\osa_msg.h"        // Pointer to messages
#include "service\osa_qmsg.h"       // Queue of pointers to messages
#include "service\osa_qsmsg.h"      // Queue of simple messages
#include "service\osa_smsg.h"       // Simple messages
#include "service\osa_tasks.h"      // Tasks
#include "service\osa_timer.h"      // System timer






























































































































//_______________________________________________________________________________________________________________
/****************************************************************************************************************/
/*                                                                                                              */
/*                     .                                                                                        */
/*                                                                                                              */
/****************************************************************************************************************/


//------------------------------------------------------------------------------
#endif

